#!/usr/bin/sh 
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").  
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#
# Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#ident	"@(#)editpkginfo.sh	1.30	07/01/24 SMI"
#
# editinfo.sh:	script to generate a 'pkginfo' file from a pkginfo.tmpl
#		file and to append comment with version, revision, 
#		product name information to commands and modules.
#
#	Usage:  see below
#
PKGINFO=pkginfo			# default name for pkginfo file
PROTOTYPE=prototype		# default name for prototype file
#
#  set default values
#
_PRODVER=1.0
_PRODREV=0.0.0
_DEVBUILD=nocomment
#
#  set local default variables with default values
#
_VERSION=${VERSION:-$_PRODVER}
_REVISION=${REVISION:-$_PRODREV}
_BUILDTYPE=${BUILDTYPE:-$_DEVBUILD}
progname=`basename $0`

usage() {
      cat <<E-O-F
$progname: INFO: Usage:
		$0 [-c cfgfile -b [devbuild insbuild relbuild] | options] \
		   [pkginfo [pkginfotempl]]

	where
		-c cfgfile	file that contains the version/packagename
				etc information

                -b [devbuild|insbuild|relbuild|nocomment]   
				development build or installation build or
				release build or no comments appended
					      
	options can be any of:
		-v ver		version of the product
 		-r rev		revision of the product
 
 	and the optional arguments can be:
 		pkginfo		the name for the produced pkginfo file
 		pkginfotempl	the 
 
  The following variables are to be replaced on the template file:
 
	%%VERSION%%		version of the current package
	%%REVISION%%		revision of the current package
	%%PSTAMP%%		package build date
	%%ARCH%%		architecture of the current package

 The variables in 'cfgfile' are expected to be in a 'sourceable' file where 
 they are set.

 If none of the options are set, the program will utilize the values found
 in the environment variables names VERSION and REVISION.

E-O-F

}
 
#   parse the arguments
#
while getopts c:b:r:v: arg
do
    case $arg in
	c)	cfgfile=$OPTARG
		;;
        b)	bldtype=$OPTARG
		;;
	r)	rev=$OPTARG
		;;
	v)	ver=$OPTARG
		;;
	\?)	usage 2>&1
		exit 2
		;;
    esac
done
if [ "$cfgfile" != "" ]
then
    if [ -f $cfgfile ]
    then
    	. /$cfgfile 
    else
	echo "$progname: ERROR: file $cfgfile does not exist." 2>&1
	exit 3
    fi
fi

#
#  Set the actual variables depending on the following order:
#
#	argument passed in a switch
#	default variable
#	sourced/environment variable
#

#
# Determine the REVISION string
#
AVS_REV=`grep ISS_VERSION_NUM= ${CODEMGR_WS}/src/sun_avs/Baseline | cut -f2 -d=`
REVISION="$AVS_REV".`date '+%Y.%m.%d'`
_REVISION=${REVISION}
_VERSION=${ver:-${_VERSION:-$VERSION}}
_REVISION=${rev:-${_REVISION:-$REVISION}}

# What type of kernel architecture
#
ARCH=`isainfo -k`
case $ARCH in
	sparc) _ARCH="sparc";;
	sparcv9) _ARCH="sparc";;
	i386) _ARCH=_"x86";;
	amd64) _ARCH="x86";;
esac

#
#	Set build stamp
_PSTAMP="`uname -n`_`date '+%m.%d.%Y'`"

shift `expr $OPTIND - 1`

PKGI=${1:-${PKGINFO}}		# optional first arg is the pkginfo name
TMPL=${2:-${PKGI}.tmpl}		# optional second arg is the template name

if [ ! -f $TMPL ]
then
    echo "$progname: ERROR: file $TMPL does not exist." 2>&1
    exit 3
fi

#
#  Replace all strings with the current values
#
sed -e "s/%%VERSION%%/$_VERSION/g" \
    -e "s/%%REVISION%%/$_REVISION/g" \
    -e "s/%%PSTAMP%%/$_PSTAMP/g" \
    -e "s/%%ARCH%%/$_ARCH/g" \
    -e '/^##/d' \
    -e 's/^ //'  $TMPL > $PKGI

#
# Edit prototype.tmpl to create 2.6 or 2.7 version of prototype.
#
PKGP=${1:-${PROTOTYPE}}
PTMPL=${2:-${PROTOTYPE}.tmpl}

if [ ! -f $PTMPL ]; then
        echo "$progname: ERROR: file $PTMPL does not exist." 2>&1
        exit 3
fi

# Set the kernel module these packages are being assembled for
#
KERNEL_MODULE=`isainfo -k`
sed -e "s/kernel_module/$KERNEL_MODULE/g" <$PTMPL >$PKGP

if [ $? -ne 0 ]; then
	echo "$progname: ERROR: could not create $PKGP file."
	exit 3
fi
#
#   Append baseline information to mcs output for each command and module.
#        
#       If this is a standard development build:
#               compress the comment section {mcs -c}
#               add the standard comment {mcs -a ${RELEASE_CM}}
#               add the development specific comment {mcs -a ${DEV_CM}}
#       If this is an installation build:
#               delete the comment section {mcs -d}
#               add the standard comment {mcs -a ${RELEASE_CM}}
#               add the development specific comment {mcs -a ${DEV_CM}}
#
#       If this is an release build:
#               delete the comment section {mcs -d}
#               add the standard comment {mcs -a ${RELEASE_CM}}
#

MCS="/usr/ccs/bin/mcs"
CMDINST="usr\/install\/bin"
PKGDEFS=$SRC/pkg/host
PROTOAREA=$ROOT
PKGNAME=`grep "^NAME=" pkginfo | awk -F= '{ print $2 }'`
POUND=`echo \\\043`
PRODNAME=`grep SUNW_PRODNAME= pkginfo | awk -F= '{ print $2 }'`
RELNUM=`echo ${_VERSION} | awk -F\. '{ print $1"."$2 }'`
RELEASE_CM="@(${POUND}) ${PRODNAME} Release ${RELNUM} Revision ${_REVISION}"
DEV_CM="@(${POUND}) ${PACK} Version ${_VERSION} Build ${VERSION}  `uname -r`"

#
# Build command to process comment depending on buildtype.
#
buildtype=${bldtype:-${_BUILDTYPE:-$BUILDTYPE}}
case ${buildtype} in
	relbuild) PROCESS_COMMENT="${MCS} -d -a '${RELEASE_CM}' ";;
	devbuild) PROCESS_COMMENT="${MCS} -c -a '${RELEASE_CM}' -a '${DEV_CM}'";;
	insbuild) PROCESS_COMMENT="${MCS} -d -a '${RELEASE_CM}' -a '${DEV_CM}'";;
	nocomment) exit ;; 
esac

#
# Use the prototype file to make a list of commands, libs and modules 
# to which a comment string will be appended.  If building 2.6 remove
# files in sparcv9 directory from list.  If building 2.7, only keep
# sparcv9 directory files.
#
/usr/bin/rm -f /tmp/pkglist*  pkglist
grep "^f " prototype | egrep -v "\." | awk '{ print $3}'> /tmp/pkglist 
grep "=" /tmp/pkglist | awk -F= '{ print $2 } '  > /tmp/pkglist1
/usr/bin/isalist | grep -cw 'sparcv9' > /dev/null 2>&1
if [ $? -ne 0 ]
then	
	grep -v sparcv9 /tmp/pkglist1 > /tmp/pkglist2
	mv /tmp/pkglist2 pkglist
else
	mv /tmp/pkglist1 pkglist
fi
for i in `cat pkglist`
do
#
# Do not place comment on a script
#
file  ${PROTOAREA}/${i} | grep script
if [ $? -ne 0 ]; then
    eval ${PROCESS_COMMENT} ${PROTOAREA}/${i} 
fi
rm -rf /tmp/*pkglist* pkglist

done 
